from portal.database.models import User, Role
from . import utils
import json

"""Basic tests to verify functionality of the API for User.

These tests do not mock the database or care about authentication/authorization.

"""


def test_create(client):
    request_dict = {
        "uco": 445574,
        "email": "baz@foo.com",
        "username": "xuser",
    }
    request_json = json.dumps(request_dict)
    response = utils.make_request(client, '/users', data=request_json, headers={"content-type": "application/json"},
                                  method='post')
    assert response.status_code == 201
    assert response.mimetype == 'application/json'

    resp_new_user = json.loads(str(response.get_data().decode("utf-8")))
    db_users = User.query.all()
    assert len(db_users) == 3
    utils.assert_user_in(db_users, resp_new_user)


def test_create_extra_key(client):
    request_dict = {
        "uco": 456789,
        "email": "baz@foo.com",
        "username": "xuser",
        "redundant key": True,
    }

    request_json = json.dumps(request_dict)
    response = utils.make_request(client, '/users', data=request_json, headers={"content-type": "application/json"},
                                  method='post')
    assert response.status_code == 201
    assert response.mimetype == 'application/json'

    resp_new_user = json.loads(str(response.get_data().decode("utf-8")))
    db_users = User.query.all()
    assert len(db_users) == 3
    utils.assert_user_in(db_users, resp_new_user)


def test_post_data_error_invalid_value(client):
    request_dict = {
        "uco": "should be a number",
        "email": "baz@foo.com",
        "username": "xuser",
    }
    request_json = json.dumps(request_dict)
    response = utils.make_request(client, '/users', data=request_json, headers={"content-type": "application/json"},
                                  method='post')
    assert response.status_code == 400
    assert response.mimetype == 'application/json'

    db_users = User.query.all()
    assert len(db_users) == 2


def test_post_data_error_missing_key(client):
    request_dict = {
        "email": "baz@foo.com",
        "username": "xuser",
    }

    request_json = json.dumps(request_dict)
    response = utils.make_request(client, '/users', data=request_json, headers={"content-type": "application/json"},
                                  method='post')
    assert response.status_code == 400
    assert response.mimetype == 'application/json'

    db_users = User.query.all()
    assert len(db_users) == 2


def test_list(client):
    response = utils.make_request(client, '/users', method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'
    data = str(response.get_data().decode("utf-8"))
    users = json.loads(data)
    assert len(users) == 2
    db_users = User.query.all()
    utils.assert_user_in(db_users, users[0])


def test_list_filter_course(client):
    response = utils.make_request(client, '/users?course=PB161', method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = str(response.get_data().decode("utf-8"))
    users = json.loads(data)
    assert len(users) == 1

    xfoo = User.query.filter_by(username="xfoo")
    utils.assert_user_in(xfoo, users[0])


def test_list_filter_group(client):
    response = utils.make_request(client, '/users?course=PB161&group=g1', method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = str(response.get_data().decode("utf-8"))
    users = json.loads(data)
    assert len(users) == 1

    xfoo = User.query.filter_by(username="xfoo")
    utils.assert_user_in(xfoo, users[0])


def test_list_filter_missing_course(client):
    response = utils.make_request(client, '/users?group=g1', method='get')
    assert response.status_code == 400
    assert response.mimetype == 'application/json'


def test_read(client):
    user = User.query.all()[0]
    response = utils.make_request(client, f'/users/{user.id}', method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = json.loads(str(response.get_data().decode("utf-8")))
    utils.assert_user(user, data)


def test_update(client):
    user = User.query.all()[0]
    request_dict = {
        "name": "new_name",
        "email": "new_email@domain.net",
        "uco": 456789,
        "is_admin": True,
    }
    request_json = json.dumps(request_dict)
    response = utils.make_request(client, f'/users/{user.id}', data=request_json,
                                  headers={"content-type": "application/json"}, method='put')

    assert response.status_code == 204
    assert response.mimetype == 'application/json'
    updated_entity = User.query.filter_by(id=user.id).first()
    assert updated_entity.name == request_dict["name"]
    assert updated_entity.email == request_dict["email"]
    assert updated_entity.uco == request_dict["uco"]
    assert updated_entity.is_admin == request_dict["is_admin"]
    assert len(User.query.all()) == 2


def test_delete(client):
    user = User.query.all()[0]

    response = utils.make_request(client, f"/users/{user.id}", method='delete')
    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    db_users = User.query.all()
    assert len(db_users) == 1
    assert user not in db_users


def test_read_submissions(client):
    user = User.query.filter_by(username="xfoo").first()

    response = utils.make_request(client, f"/users/{user.id}/submissions", method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'
    # check if the result is paginated?
    data = str(response.get_data().decode("utf-8"))
    submissions = json.loads(data)
    assert len(submissions) == 1
    utils.assert_submission(user.submissions[0], submissions[0])


def test_read_submissions_filter_course(client):
    user = User.query.filter_by(username="xfoo").first()

    response = utils.make_request(client, f"/users/{user.id}/submissions?course=PB161", method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = str(response.get_data().decode("utf-8"))
    submissions = json.loads(data)
    assert len(submissions) == 1
    utils.assert_submission(user.submissions[0], submissions[0])


def test_read_submissions_filter_course_project(client):
    user = User.query.filter_by(username="xfoo").first()

    response = utils.make_request(client, f"/users/{user.id}/submissions?course=PB161&project=p1", method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = str(response.get_data().decode("utf-8"))
    submissions = json.loads(data)
    assert len(submissions) == 1
    utils.assert_submission(user.submissions[0], submissions[0])


def test_read_submissions_filter_missing_course(client):
    user = User.query.filter_by(username="xfoo").first()

    response = utils.make_request(client, f"/users/{user.id}/submissions?project=p1", method='get')
    assert response.status_code == 400
    assert response.mimetype == 'application/json'


def test_read_roles(client):
    user = User.query.filter_by(username="xfoo").first()
    response = utils.make_request(client, f"/users/{user.id}/roles", method='get')

    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = str(response.get_data().decode("utf-8"))
    user_roles = json.loads(data)
    assert len(user_roles) == 1
    assert len(user.roles) == 1
    # rewrite to not use user.roles
    utils.assert_role(user.roles[0], user_roles[0])


def test_read_roles_filter_course(client):
    user = User.query.filter_by(username="xfoo").first()
    response = utils.make_request(client, f"/users/{user.id}/roles?course=PB161", method='get')

    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = str(response.get_data().decode("utf-8"))
    user_roles = json.loads(data)
    assert len(user_roles) == 1
    assert len(user.roles) == 1
    # rewrite to not use user.roles
    utils.assert_role(user.roles[0], user_roles[0])


def test_read_courses(client):
    user = User.query.filter_by(username="xfoo").first()

    response = utils.make_request(client, f"/users/{user.id}/courses", method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = str(response.get_data().decode("utf-8"))
    user_courses = json.loads(data)
    assert len(user_courses) == 1

    roles = Role.query.all()
    roles_filtered = [role for role in roles if user in role.users]

    courses = [role.course for role in roles_filtered]
    assert len(courses) == 1
    utils.assert_course(courses[0], user_courses[0])


def test_read_groups(client):
    user = User.query.filter_by(username="xfoo").first()
    response = utils.make_request(client, f"/users/{user.id}/groups", method='get')

    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = str(response.get_data().decode("utf-8"))
    user_groups = json.loads(data)
    assert len(user_groups) == 1
    assert len(user.groups) == 1
    # rewrite to not use user.groups
    utils.assert_group(user.groups[0], user_groups[0])


def test_read_groups_filter_course(client):
    user = User.query.filter_by(username="xfoo").first()
    response = utils.make_request(client, f"/users/{user.id}/groups?course=PB161", method='get')

    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = str(response.get_data().decode("utf-8"))
    user_groups = json.loads(data)
    assert len(user_groups) == 1
    assert len(user.groups) == 1
    # rewrite to not use user.groups
    utils.assert_group(user.groups[0], user_groups[0])


def test_user_reviews(client):
    user = User.query.filter_by(username="xfoo").first()
    response = utils.make_request(client, f"/users/{user.id}/reviews", method='get')

    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = str(response.get_data().decode("utf-8"))
    user_reviews = json.loads(data)
    assert len(user_reviews) == 1
    # the test is incomplete - only print
    print(f"user_reviews: {user_reviews}")
